Struct Xp

pub struct Xp<T: Config<I>, I: 'static> {
    pub free: T::Xp,
    pub reserve: T::Xp,
    pub lock: T::Xp,
    pub pulse: Accumulator<T, I>,
    pub timestamp: BlockNumberFor<T>,
}
The main XP data structure that is utilized on implementation of XpSystem::Xp.

It provides a high-level detail for managing liquid points, reserved points, locked points, and reputation-based pulse tracking with timestamp information.

Point Categories

  • Free Points: Liquid XP that the owner can freely access and use.
  • Reserved Points: XP temporarily set aside for Runtime specific purposes.
  • Locked Points: XP that is restricted by the Runtime provider (other pallets) or implementor.

Reputation System

  • Pulse: A discrete accumulator that tracks XP mutation frequency for reputation.
  • Timestamp: Block number of the last XP increment for heartbeat tracking.

Fields

free: T::Xp

Liquid XP points that the owner can freely access.

reserve: T::Xp

Reserved XP points that are temporarily set aside for specific purposes.

This field aggregates all reserved XP across different RuntimeReasons, enabling efficient access to the total reserved balance without needing to iterate through individual reservation records.

lock: T::Xp

Locked XP points that are restricted by the Runtime provider or implementor.

This field aggregates all locked XP across different RuntimeReasons, enabling efficient access to the total locked balance without needing to iterate through individual lock records.

pulse: Accumulator<T, I>

Reputation-based pulse accumulator that tracks XP mutation frequency.

The provider represents the runtime intent, awarding XP based on completed work.

The pulse acts as a “heartbeat” of XP activity, serving as a reputational metric for each XP account. It is proportional to the frequency and amount of XP increments, and may influence the raw XP awarded for future actions.

In an untrusted environment, pulse is used as a reputational resource, allowing the system to adjust raw XP based on the quality and consistency of the account’s activity as reflected by its pulse.

timestamp: BlockNumberFor<T>

The block number at which XP was last incremented.

This timestamp is used to identify inactive (“dead”) XP accounts and can be leveraged to conditionally determine whether to increase reputation (i.e., pulse) based on recent activity, or if the XP is subjected to reaping procedures.