pub struct XpProgress<T: Config<I>, I: 'static> {
pub level: T::Pulse,
pub progress: T::Pulse,
pub threshold: T::Pulse,
pub per_action: T::Pulse,
}Expand description
Represents the progression mechanics behind XP scaling.
Exposes the current level, progress toward the next increment, and the parameters that control how progress is accumulated.
Fields
level: T::PulseCurrent multiplier level.
progress: T::PulseProgress toward the next level.
threshold: T::PulseTotal progress required to reach the next level.
per_action: T::PulseProgress gained per earn_xp action.